@@ -22,9 +22,7 @@ class CKANAPIError(Exception):
         return repr(self.args)
 
 try:
-    from ckan.logic import (ParameterError, NotAuthorized, NotFound,
-                            ValidationError)
-    from ckan.lib.search import SearchQueryError, SearchError
+    import ckan
 
 except ImportError:
     # Implement the minimum to be compatible with existing errors
@@ -57,6 +55,12 @@ except ImportError:
     class SearchError(CKANAPIError):
         pass
 
+else:
+    # import ckan worked, so these must not fail
+    from ckan.logic import (ParameterError, NotAuthorized, NotFound,
+                            ValidationError)
+    from ckan.lib.search import SearchQueryError, SearchError
+
 
 class ActionShortcut(object):
     """
@@ -162,15 +166,9 @@ class RemoteCKAN(object):
         function will convert it back to an exception that matches the
         one the action function itself raised.
         """
-        if not data_dict:
-            data_dict = {}
-        data = json.dumps(data_dict)
-        header = {'Content-Type': 'application/json'}
-        if self.api_key:
-            header['Authorization'] = self.api_key
-        url = self.address + '/api/action/' + action
-        status, response = self._request_fn(url, data, header)
-        return reverse_apicontroller_action(response, status)
+        url, data, headers = prepare_action(action, data_dict, self.api_key)
+        status, response = self._request_fn(self.address + url, data, headers)
+        return reverse_apicontroller_action(status, response)
 
     def _request_fn(self, url, data, headers):
         req = urllib2.Request(url, data, headers)
@@ -181,14 +179,58 @@ class RemoteCKAN(object):
             return e.code, e.read()
 
 
-def reverse_apicontroller_action(response, status):
+class TestAppCKAN(object):
+    """
+    An interface to the the CKAN API actions on a paste TestApp
+
+    :param test_app: the paste.fixture.TestApp instance, stored as
+                    self.test_app
+    :param api_key: the API key to pass as an 'Authorization' header
+                    when actions are called, stored as self.api_key
+    """
+    def __init__(self, test_app, api_key=None):
+        self.test_app = test_app
+        self.api_key = api_key
+        self.action = ActionShortcut(self)
+
+    def call_action(self, action, data_dict=None):
+        """
+        :param action: the action name, e.g. 'package_create'
+        :param data_dict: the dict to pass to the action as JSON,
+                          defaults to {}
+
+        This function parses the response from the server as JSON and
+        returns the decoded value.  When an error is returned this
+        function will convert it back to an exception that matches the
+        one the action function itself raised.
+        """
+        url, data, headers = prepare_action(action, data_dict, self.api_key)
+        r = self.test_app.post(url, data, headers)
+        return reverse_apicontroller_action(r.status, r.body)
+
+
+def prepare_action(action, data_dict=None, api_key=None):
+    """
+    Return action_url, data_json, http_headers
+    """
+    if not data_dict:
+        data_dict = {}
+    data = json.dumps(data_dict)
+    headers = {'Content-Type': 'application/json'}
+    if api_key:
+        headers['Authorization'] = api_key
+    url = '/api/action/' + action
+    return url, data, headers
+
+
+def reverse_apicontroller_action(status, response):
     """
     Make an API call look like a direct action call by reversing the
     exception -> HTTP response translation that APIController.action does
     """
     try:
         parsed = json.loads(response)
-        if status == 200:
+        if parsed.get('success'):
             return parsed
         if hasattr(parsed, 'get'):
             err = parsed.get('error', {})
